import requests
from requests import HTTPError
from .base import BaseClient
from .exceptions import ClientAPIException, ClientNotFoundError


class RemoteIdentificationClient(BaseClient):
    ROUTE_CUSTOMER_INFO = 'customer-info'
    ROUTE_CUSTOMER_PASSPORT = 'customer-photo'
    ROUTE_CUSTOMER_SELFIE = 'customer-selfie'

    def get_user(
            self,
            phone: str,
            do_print: bool = True
    ) -> dict:
        """
            Вытаскивает данные пользователя с УИ,
            если не найдено кидает ошибку ClientNotFoundError
        """
        phone = self._strip_phone(phone)
        data = self._request('get', f'{self.ROUTE_CUSTOMER_INFO}/{phone}', {}, do_print)
        return self._check_data_with_status(data)

    def get_passport_photos(
            self,
            phone,
            do_print: bool = True
    ) -> dict:
        """
            Вытаскивает фото паспорта (passportForwardPhoto, passportBackPhoto, facePhoto),
            если не найдено кидает ошибку ClientNotFoundError
        """
        phone = self._strip_phone(phone)
        data = self._request('get', f'{self.ROUTE_CUSTOMER_PASSPORT}/{phone}', {}, do_print)
        return self._check_data_with_status(data)

    def get_selfie(
            self,
            phone,
            do_print: bool = True
    ) -> str:
        """
            Вытаскивает селфи с УИ, если не найдено кидает ошибку ClientNotFoundError
        """
        phone = self._strip_phone(phone)
        data = self._request('get', f'{self.ROUTE_CUSTOMER_SELFIE}/{phone}', {}, do_print)
        return self._check_data_with_status(data).get('selfie')

    @staticmethod
    def _check_data_with_status(data) -> dict:
        if data.get('status', None):
            if data['status'] == 1:
                return data
        raise ClientNotFoundError(404, 'Не найдено')

    def _request(self, method, route, data, do_print=True, headers=None):
        if not headers:
            headers = {'Content-Type': 'application/json', 'Accept': 'application/json'}
        headers.update({"holy": self._token})

        response = None
        try:
            response = requests.request(
                method,
                self._build_absolute_url(route, data),
                headers=headers
            )
        except HTTPError:
            raise ClientAPIException(response.status_code, response.text)
        self._checker_response(response)

        if do_print:
            print(response.json())
        return response.json()



import requests
from requests import HTTPError
from .base import BaseClient
from .exceptions import ClientAPIException, ClientNotFoundError


class MBankClient(BaseClient):
    ROUTE_CHECK_MBANK = 'check'

    def _request(self, method, route, data, do_print=True, headers=None):
        if not headers:
            headers = {'Content-Type': 'application/json', 'Accept': 'application/json'}
        headers.update({"authenticate": self._token})

        response = None
        try:
            response = requests.request(
                method,
                self._build_absolute_url(route, data),
                headers=headers
            )
        except HTTPError:
            raise ClientAPIException(response.status_code, response.text)
        self._checker_response(response)

        if do_print:
            print(response.json())
        return response.json()

    def check_mbank_exist(self, phone, do_print=True):
        """
            Проверяет наличие Мбанк по номеру телефона
            Check if payer exists
            field status of response json can be in two states:
            0 - payer exists in the system
            114 - payer not found
        """
        phone = self._strip_phone(phone)
        data = {'phone': phone}
        data = self._request('get', f'{self.ROUTE_CHECK_MBANK}', data, do_print)
        return self._check_data_with_code(data)

    @staticmethod
    def _check_data_with_code(data) -> dict:
        if 'code' in data:
            if data.get('code', 114) == 0:
                return data
        raise ClientNotFoundError(404, 'Не найдено')



import abc
from urllib.parse import urljoin, urlencode

import requests

from .exceptions import (
    ClientNotFoundError,
    ClientBadRequestError,
    ClientServerError,
    ClientAPIException
)


class BaseClient(abc.ABC):
    def __init__(self, token: str, url: str):
        self._token = token
        self._main_url = url

    @staticmethod
    def _strip_phone(phone):
        return str(phone).strip('+')

    def _build_absolute_url(self, url, data):
        return f'{urljoin(self._main_url, url)}?{urlencode(data)}'

    @staticmethod
    def _checker_response(response: requests.Response):
        status_code = response.status_code
        if (
                status_code == 200 or
                status_code == 201 or
                status_code == 202
        ):
            return

        if status_code == 404:
            raise ClientNotFoundError(response.status_code, response.text)

        if 400 <= status_code < 500:
            raise ClientBadRequestError(response.status_code, response.text)

        if status_code >= 500:
            raise ClientServerError(response.status_code, response.text)
        raise ClientAPIException(response.status_code, response.text)



class ClientAPIException(Exception):
    def __init__(self, code: int, reason: str) -> None:
        self.error_code = code
        self.reason = reason


class ClientNotFoundError(ClientAPIException):
    ...


class ClientBadRequestError(ClientAPIException):
    ...


class ClientServerError(ClientAPIException):
    ...



from setuptools import setup, find_packages

setup(
    name='mbank_service',
    version='0.1.2',
    include_package_data=True,
    packages=find_packages(),
    license='TBD',
    author='Eldiyar Kandzhebaev',
    python_requires='>=3.9',
)
